﻿using Infragistics.Win;
using Infragistics.Win.Misc;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace _155471
{
    public class NotificationFilter:IUIElementDrawFilter
    {
        const int maxNotificationSize = 24;
        private Rectangle GetRect(Rectangle rectangle)
        {
            Rectangle rect = rectangle;
            int side = Math.Min(Math.Min(rect.Width / 2, rect.Height / 2), maxNotificationSize);
            Point location = new Point(rect.Right, rect.Y);
            location.Offset(-(side + 2), 1);
            return new Rectangle(location, new Size(side, side));
        }

        private float FindSize(System.Drawing.Graphics g, string text, Size Room, Font PreferedFont)
        {
            using (Font f = new Font(PreferedFont.FontFamily, PreferedFont.Size, FontStyle.Bold))
            {
                int lineSpacing = f.FontFamily.GetLineSpacing(FontStyle.Bold);
                float SizeInPixel = f.Size * lineSpacing / f.FontFamily.GetEmHeight(FontStyle.Bold);
                SizeF RealSize = g.MeasureString(text, f);

                float HeightScaleRatio = Room.Height / RealSize.Height;
                float WidthScaleRatio = Room.Width / RealSize.Width;
                float ScaleRatio = Math.Max(HeightScaleRatio, WidthScaleRatio);
                float ScaleFontSize = SizeInPixel * ScaleRatio;
                ScaleFontSize = (float)Math.Min(Room.Height * 0.7, ScaleFontSize);

                return ScaleFontSize;
            }
        }

        public GraphicsPath GetShapeFor(Rectangle rect)
        {
            var path = new GraphicsPath();
            path.StartFigure();
            path.AddEllipse(rect);
            path.CloseAllFigures();
            return path;
        }

        public Hashtable NotificationString = new Hashtable();

        #region IUIElementDrawFilter

        public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            string message = NotificationString[drawParams.Element.Control.Name] as string;
            if (string.IsNullOrEmpty(message))
                return false;

            Rectangle rect = GetRect(drawParams.InvalidRect);
            rect.Offset(-1, 1);
            Rectangle sheduleRect = rect;
            sheduleRect.Inflate(1, 1);
            sheduleRect.Offset(0, 1);

            drawParams.Graphics.CompositingQuality = CompositingQuality.HighQuality;
            //draw shadow 
            using (GraphicsPath path = this.GetShapeFor(sheduleRect))
            {
                drawParams.AppearanceData.BackColor = Color.FromArgb(100, Color.Black);
                drawParams.Graphics.FillPath(drawParams.BackBrush, path);
            }

            //Draw outline 
            using (GraphicsPath path = this.GetShapeFor(rect))
            {
                drawParams.AppearanceData.BackColor = Color.White;
                drawParams.Graphics.FillPath(drawParams.BackBrush, path);
            }

            //Draw Inner
            rect.Inflate(-2, -2);
            using (GraphicsPath path = this.GetShapeFor(rect))
            {
                drawParams.AppearanceData.BackColor = Color.Red;
                drawParams.Graphics.FillPath(drawParams.BackBrush, path);
            }

            var size = FindSize(drawParams.Graphics, message, rect.Size, drawParams.Font);

            using (Font f = new Font(drawParams.Font.Name, size, FontStyle.Bold, GraphicsUnit.Pixel))
            {
                drawParams.Graphics.DrawString(message, f, Brushes.White, rect, new StringFormat() { Alignment = StringAlignment.Center, LineAlignment = StringAlignment.Center });
            }

            return false;
        }

        public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            if (drawParams.Element is UltraButtonControlUIElement && NotificationString.ContainsKey(drawParams.Element.Control.Name))
                return DrawPhase.AfterDrawElement;
            return DrawPhase.None;

        }

        #endregion IUIElementDrawFilter
    }
}
